@@ -398,6 +398,11 @@ bool XcbWindow::GetNativeHandle(HandleType type, void** handle)
     }
 }
 
+std::string XcbWindow::GetWsiExtension() const
+{
+    return VK_KHR_XCB_SURFACE_EXTENSION_NAME;
+}
+
 VkResult
 XcbWindow::CreateSurface(const encode::InstanceTable* table, VkInstance instance, VkFlags flags, VkSurfaceKHR* pSurface)
 {
