@@ -37,9 +37,15 @@ GFXRECON_BEGIN_NAMESPACE(decode)
 
 FileProcessor::FileProcessor() :
     file_header_{}, file_descriptor_(nullptr), current_frame_number_(0), bytes_read_(0),
-    error_state_(kErrorInvalidFileDescriptor), annotation_handler_(nullptr), compressor_(nullptr), block_index_(0)
+    error_state_(kErrorInvalidFileDescriptor), annotation_handler_(nullptr), compressor_(nullptr), block_index_(0),
+    api_call_index_(0), block_limit_(0)
 {}
 
+FileProcessor::FileProcessor(uint64_t block_limit) : FileProcessor()
+{
+    block_limit_ = block_limit;
+}
+
 FileProcessor::~FileProcessor()
 {
     if (nullptr != compressor_)
@@ -122,14 +128,52 @@ bool FileProcessor::ProcessAllFrames()
 {
     bool success = true;
 
+    block_index_ = 0;
+
     while (success)
     {
-        success = ProcessNextFrame();
+        if (success)
+        {
+            success = ProcessNextFrame();
+        }
     }
 
     return (error_state_ == kErrorNone);
 }
 
+bool FileProcessor::ContinueDecoding()
+{
+    bool early_exit = false;
+    // If a block limit was specified, obey it.
+    // If not (block_limit_ = 0),  then the consumer may determine early exit
+    if (block_limit_ > 0)
+    {
+        if (block_index_ > block_limit_)
+        {
+            early_exit = true;
+        }
+    }
+    else
+    {
+        int completed_decoders = 0;
+
+        for (auto& decoder : decoders_)
+        {
+            if (decoder->IsComplete(block_index_) == true)
+            {
+                completed_decoders++;
+            }
+        }
+
+        if (completed_decoders == decoders_.size())
+        {
+            early_exit = true;
+        }
+    }
+
+    return !early_exit;
+}
+
 bool FileProcessor::ProcessFileHeader()
 {
     bool success = false;
@@ -195,108 +239,144 @@ bool FileProcessor::ProcessBlocks()
 
     while (success)
     {
-        success = ReadBlockHeader(&block_header);
+        success = ContinueDecoding();
 
         if (success)
         {
-            if (format::RemoveCompressedBlockBit(block_header.type) == format::BlockType::kFunctionCallBlock)
+            success = ReadBlockHeader(&block_header);
+
+            for (auto decoder : decoders_)
             {
-                format::ApiCallId api_call_id = format::ApiCallId::ApiCall_Unknown;
-
-                success = ReadBytes(&api_call_id, sizeof(api_call_id));
-
-                if (success)
-                {
-                    success = ProcessFunctionCall(block_header, api_call_id);
-
-                    // Break from loop on frame delimiter.
-                    if (IsFrameDelimiter(api_call_id))
-                    {
-                        // Make sure to increment the frame number on the way out.
-                        ++current_frame_number_;
-                        ++block_index_;
-                        break;
-                    }
-                }
-                else
-                {
-                    HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read function call block header");
-                }
+                decoder->SetCurrentBlockIndex(block_index_);
             }
-            else if (format::RemoveCompressedBlockBit(block_header.type) == format::BlockType::kMetaDataBlock)
+
+            if (success)
             {
-                format::MetaDataId meta_data_id = format::MakeMetaDataId(format::ApiFamilyId::ApiFamily_None,
-                                                                         format::MetaDataType::kUnknownMetaDataType);
-
-                success = ReadBytes(&meta_data_id, sizeof(meta_data_id));
-
-                if (success)
+                if (format::RemoveCompressedBlockBit(block_header.type) == format::BlockType::kFunctionCallBlock)
                 {
-                    success = ProcessMetaData(block_header, meta_data_id);
-                }
-                else
-                {
-                    HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read meta-data block header");
-                }
-            }
-            else if (block_header.type == format::BlockType::kStateMarkerBlock)
-            {
-                format::MarkerType marker_type  = format::MarkerType::kUnknownMarker;
-                uint64_t           frame_number = 0;
+                    format::ApiCallId api_call_id = format::ApiCallId::ApiCall_Unknown;
 
-                success = ReadBytes(&marker_type, sizeof(marker_type));
-
-                if (success)
-                {
-                    success = ProcessStateMarker(block_header, marker_type);
-                }
-                else
-                {
-                    HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read state marker header");
-                }
-            }
-            else if (block_header.type == format::BlockType::kAnnotation)
-            {
-                if (annotation_handler_ != nullptr)
-                {
-                    format::AnnotationType annotation_type = format::AnnotationType::kUnknown;
-
-                    success = ReadBytes(&annotation_type, sizeof(annotation_type));
+                    success = ReadBytes(&api_call_id, sizeof(api_call_id));
 
                     if (success)
                     {
-                        success = ProcessAnnotation(block_header, annotation_type);
+                        success = ProcessFunctionCall(block_header, api_call_id);
+
+                        // Break from loop on frame delimiter.
+                        if (IsFrameDelimiter(api_call_id))
+                        {
+                            // Make sure to increment the frame number on the way out.
+                            ++current_frame_number_;
+                            ++block_index_;
+                            break;
+                        }
                     }
                     else
                     {
-                        HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read annotation block header");
+                        HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read function call block header");
+                    }
+                }
+                else if (format::RemoveCompressedBlockBit(block_header.type) == format::BlockType::kMethodCallBlock)
+                {
+                    format::ApiCallId api_call_id = format::ApiCallId::ApiCall_Unknown;
+
+                    success = ReadBytes(&api_call_id, sizeof(api_call_id));
+
+                    if (success)
+                    {
+                        success = ProcessMethodCall(block_header, api_call_id);
+
+                        // Break from loop on frame delimiter.
+                        if (IsFrameDelimiter(api_call_id))
+                        {
+                            // Make sure to increment the frame number on the way out.
+                            ++current_frame_number_;
+                            ++block_index_;
+                            break;
+                        }
+                    }
+                    else
+                    {
+                        HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read function call block header");
+                    }
+                }
+                else if (format::RemoveCompressedBlockBit(block_header.type) == format::BlockType::kMetaDataBlock)
+                {
+                    format::MetaDataId meta_data_id = format::MakeMetaDataId(
+                        format::ApiFamilyId::ApiFamily_None, format::MetaDataType::kUnknownMetaDataType);
+
+                    success = ReadBytes(&meta_data_id, sizeof(meta_data_id));
+
+                    if (success)
+                    {
+                        success = ProcessMetaData(block_header, meta_data_id);
+                    }
+                    else
+                    {
+                        HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read meta-data block header");
+                    }
+                }
+                else if (block_header.type == format::BlockType::kStateMarkerBlock)
+                {
+                    format::MarkerType marker_type  = format::MarkerType::kUnknownMarker;
+                    uint64_t           frame_number = 0;
+
+                    success = ReadBytes(&marker_type, sizeof(marker_type));
+
+                    if (success)
+                    {
+                        success = ProcessStateMarker(block_header, marker_type);
+                    }
+                    else
+                    {
+                        HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read state marker header");
+                    }
+                }
+                else if (block_header.type == format::BlockType::kAnnotation)
+                {
+                    if (annotation_handler_ != nullptr)
+                    {
+                        format::AnnotationType annotation_type = format::AnnotationType::kUnknown;
+
+                        success = ReadBytes(&annotation_type, sizeof(annotation_type));
+
+                        if (success)
+                        {
+                            success = ProcessAnnotation(block_header, annotation_type);
+                        }
+                        else
+                        {
+                            HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read annotation block header");
+                        }
+                    }
+                    else
+                    {
+                        // If there is no annotation handler to process the annotation, we can skip the annotation
+                        // block.
+                        GFXRECON_CHECK_CONVERSION_DATA_LOSS(size_t, block_header.size);
+                        success = SkipBytes(static_cast<size_t>(block_header.size));
                     }
                 }
                 else
                 {
-                    // If there is no annotation handler to process the annotation, we can skip the annotation block.
+                    // Unrecognized block type.
+                    GFXRECON_LOG_WARNING("Skipping unrecognized file block with type %u", block_header.type);
                     GFXRECON_CHECK_CONVERSION_DATA_LOSS(size_t, block_header.size);
                     success = SkipBytes(static_cast<size_t>(block_header.size));
                 }
             }
             else
             {
-                // Unrecognized block type.
-                GFXRECON_LOG_WARNING("Skipping unrecognized file block with type %u", block_header.type);
-                GFXRECON_CHECK_CONVERSION_DATA_LOSS(size_t, block_header.size);
-                success = SkipBytes(static_cast<size_t>(block_header.size));
-            }
-        }
-        else
-        {
-            if (!feof(file_descriptor_))
-            {
-                // No data has been read for the current block, so we don't use 'HandleBlockReadError' here, as it
-                // assumes that the block header has been successfully read and will print an incomplete block at end
-                // of file warning when the file is at EOF without an error. For this case (the normal EOF case) we
-                // print nothing at EOF, or print an error message and set the error code directly when not at EOF.
-                GFXRECON_LOG_ERROR("Failed to read block header");
-                error_state_ = kErrorReadingBlockHeader;
+                if (!feof(file_descriptor_))
+                {
+                    // No data has been read for the current block, so we don't use 'HandleBlockReadError' here, as it
+                    // assumes that the block header has been successfully read and will print an incomplete block at
+                    // end of file warning when the file is at EOF without an error. For this case (the normal EOF case)
+                    // we print nothing at EOF, or print an error message and set the error code directly when not at
+                    // EOF.
+                    GFXRECON_LOG_ERROR("Failed to read block header");
+                    error_state_ = kErrorReadingBlockHeader;
+                }
             }
         }
         ++block_index_;
@@ -466,6 +546,84 @@ bool FileProcessor::ProcessFunctionCall(const format::BlockHeader& block_header,
     return success;
 }
 
+bool FileProcessor::ProcessMethodCall(const format::BlockHeader& block_header, format::ApiCallId call_id)
+{
+    size_t           parameter_buffer_size = static_cast<size_t>(block_header.size) - sizeof(call_id);
+    uint64_t         uncompressed_size     = 0;
+    format::HandleId object_id             = 0;
+    ApiCallInfo      call_info             = {};
+
+    bool success = ReadBytes(&object_id, sizeof(object_id));
+    success      = success && ReadBytes(&call_info.thread_id, sizeof(call_info.thread_id));
+
+    if (success)
+    {
+        parameter_buffer_size -= (sizeof(object_id) + sizeof(call_info.thread_id));
+
+        if (format::IsBlockCompressed(block_header.type))
+        {
+            parameter_buffer_size -= sizeof(uncompressed_size);
+            success = ReadBytes(&uncompressed_size, sizeof(uncompressed_size));
+
+            if (success)
+            {
+                GFXRECON_CHECK_CONVERSION_DATA_LOSS(size_t, uncompressed_size);
+
+                size_t actual_size = 0;
+                success            = ReadCompressedParameterBuffer(
+                    parameter_buffer_size, static_cast<size_t>(uncompressed_size), &actual_size);
+
+                if (success)
+                {
+                    assert(actual_size == uncompressed_size);
+                    parameter_buffer_size = static_cast<size_t>(uncompressed_size);
+                }
+                else
+                {
+                    HandleBlockReadError(kErrorReadingCompressedBlockData,
+                                         "Failed to read compressed function call block data");
+                }
+            }
+            else
+            {
+                HandleBlockReadError(kErrorReadingCompressedBlockHeader,
+                                     "Failed to read compressed function call block header");
+            }
+        }
+        else
+        {
+            success = ReadParameterBuffer(parameter_buffer_size);
+
+            if (!success)
+            {
+                HandleBlockReadError(kErrorReadingBlockData, "Failed to read function call block data");
+            }
+        }
+
+        if (success)
+        {
+            for (auto decoder : decoders_)
+            {
+                if (decoder->SupportsApiCall(call_id))
+                {
+                    DecodeAllocator::Begin();
+                    decoder->DecodeMethodCall(
+                        call_id, object_id, call_info, parameter_buffer_.data(), parameter_buffer_size);
+                    DecodeAllocator::End();
+                }
+            }
+
+            ++api_call_index_;
+        }
+    }
+    else
+    {
+        HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read function call block header");
+    }
+
+    return success;
+}
+
 bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, format::MetaDataId meta_data_id)
 {
     bool success = false;
@@ -531,6 +689,53 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
             HandleBlockReadError(kErrorReadingBlockHeader, "Failed to read fill memory meta-data block header");
         }
     }
+    else if (meta_data_type == format::MetaDataType::kFillMemoryResourceValueCommand)
+    {
+        format::FillMemoryResourceValueCommandHeader header;
+
+        success = ReadBytes(&header.thread_id, sizeof(header.thread_id));
+        success = ReadBytes(&header.resource_value_count, sizeof(header.resource_value_count));
+
+        if (success)
+        {
+            uint64_t data_size = header.resource_value_count * (sizeof(format::ResourceValueType) + sizeof(uint64_t));
+            GFXRECON_CHECK_CONVERSION_DATA_LOSS(size_t, data_size);
+
+            if (format::IsBlockCompressed(block_header.type))
+            {
+                size_t uncompressed_size = 0;
+                size_t compressed_size   = static_cast<size_t>(block_header.size) - sizeof(meta_data_id) -
+                                         sizeof(header.thread_id) - sizeof(header.resource_value_count);
+                size_t uncompressed_data = static_cast<size_t>(data_size);
+                success = ReadCompressedParameterBuffer(compressed_size, uncompressed_data, &uncompressed_size);
+            }
+            else
+            {
+                success = ReadParameterBuffer(static_cast<size_t>(data_size));
+            }
+
+            if (success)
+            {
+                for (auto decoder : decoders_)
+                {
+                    if (decoder->SupportsMetaDataId(meta_data_id))
+                    {
+                        decoder->DispatchFillMemoryResourceValueCommand(header, parameter_buffer_.data());
+                    }
+                }
+            }
+            else
+            {
+                HandleBlockReadError(kErrorReadingBlockData,
+                                     "Failed to read fill memory resource value meta-data block");
+            }
+        }
+        else
+        {
+            HandleBlockReadError(kErrorReadingBlockHeader,
+                                 "Failed to read fill memory resource value meta-data block header");
+        }
+    }
     else if (meta_data_type == format::MetaDataType::kResizeWindowCommand)
     {
         // This command does not support compression.
@@ -588,6 +793,49 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
             HandleBlockReadError(kErrorReadingBlockData, "Failed to read resize window 2 meta-data block");
         }
     }
+    else if (meta_data_type == format::MetaDataType::kExeFileInfoCommand)
+    {
+        format::ExeFileInfoBlock header;
+        success = ReadBytes(&header.thread_id, sizeof(header.thread_id));
+
+        success =
+            success && ReadBytes(&header.info_record.ProductVersion, gfxrecon::util::filepath::kMaxFilePropertySize);
+        success = success && ReadBytes(&header.info_record.FileVersion, gfxrecon::util::filepath::kMaxFilePropertySize);
+        success = success && ReadBytes(&header.info_record.AppVersion,
+                                       sizeof(uint32_t) * gfxrecon::util::filepath::kFileVersionSize);
+        success = success && ReadBytes(&header.info_record.AppName, gfxrecon::util::filepath::kMaxFilePropertySize);
+        success = success && ReadBytes(&header.info_record.CompanyName, gfxrecon::util::filepath::kMaxFilePropertySize);
+        success =
+            success && ReadBytes(&header.info_record.FileDescription, gfxrecon::util::filepath::kMaxFilePropertySize);
+        success =
+            success && ReadBytes(&header.info_record.InternalName, gfxrecon::util::filepath::kMaxFilePropertySize);
+        success =
+            success && ReadBytes(&header.info_record.OriginalFilename, gfxrecon::util::filepath::kMaxFilePropertySize);
+        success = success && ReadBytes(&header.info_record.ProductName, gfxrecon::util::filepath::kMaxFilePropertySize);
+
+        if (success)
+        {
+            for (auto decoder : decoders_)
+            {
+                decoder->DispatchExeFileInfo(header.thread_id, header);
+            }
+        }
+    }
+    else if (meta_data_type == format::MetaDataType::kDriverInfoCommand)
+    {
+        format::DriverInfoBlock header;
+        success = ReadBytes(&header.thread_id, sizeof(header.thread_id));
+
+        success = success && ReadBytes(&header.driver_record, gfxrecon::util::filepath::kMaxDriverInfoSize);
+
+        if (success)
+        {
+            for (auto decoder : decoders_)
+            {
+                decoder->DispatchDriverInfo(header.thread_id, header);
+            }
+        }
+    }
     else if (meta_data_type == format::MetaDataType::kDisplayMessageCommand)
     {
         // This command does not support compression.
@@ -796,6 +1044,30 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
             HandleBlockReadError(kErrorReadingBlockData, "Failed to read destroy hardware buffer meta-data block");
         }
     }
+    else if (meta_data_type == format::MetaDataType::kCreateHeapAllocationCommand)
+    {
+        // This command does not support compression.
+        assert(block_header.type != format::BlockType::kCompressedMetaDataBlock);
+
+        format::CreateHeapAllocationCommand header;
+
+        success = ReadBytes(&header.thread_id, sizeof(header.thread_id));
+        success = success && ReadBytes(&header.allocation_id, sizeof(header.allocation_id));
+        success = success && ReadBytes(&header.allocation_size, sizeof(header.allocation_size));
+
+        if (success)
+        {
+            for (auto decoder : decoders_)
+            {
+                decoder->DispatchCreateHeapAllocationCommand(
+                    header.thread_id, header.allocation_id, header.allocation_size);
+            }
+        }
+        else
+        {
+            HandleBlockReadError(kErrorReadingBlockData, "Failed to read create heap allocation meta-data block");
+        }
+    }
     else if (meta_data_type == format::MetaDataType::kSetDevicePropertiesCommand)
     {
         // This command does not support compression.
@@ -1220,6 +1492,222 @@ bool FileProcessor::ProcessMetaData(const format::BlockHeader& block_header, for
             }
         }
     }
+    else if (meta_data_type == format::MetaDataType::kInitSubresourceCommand)
+    {
+        format::InitSubresourceCommandHeader header;
+
+        success = ReadBytes(&header.thread_id, sizeof(header.thread_id));
+        success = success && ReadBytes(&header.device_id, sizeof(header.device_id));
+        success = success && ReadBytes(&header.resource_id, sizeof(header.resource_id));
+        success = success && ReadBytes(&header.subresource, sizeof(header.subresource));
+        success = success && ReadBytes(&header.initial_state, sizeof(header.initial_state));
+        success = success && ReadBytes(&header.resource_state, sizeof(header.resource_state));
+        success = success && ReadBytes(&header.barrier_flags, sizeof(header.barrier_flags));
+        success = success && ReadBytes(&header.data_size, sizeof(header.data_size));
+
+        if (success)
+        {
+            GFXRECON_CHECK_CONVERSION_DATA_LOSS(size_t, header.data_size);
+
+            if (format::IsBlockCompressed(block_header.type))
+            {
+                size_t uncompressed_size = 0;
+                size_t compressed_size =
+                    static_cast<size_t>(block_header.size) - (sizeof(header) - sizeof(header.meta_header.block_header));
+
+                success = ReadCompressedParameterBuffer(
+                    compressed_size, static_cast<size_t>(header.data_size), &uncompressed_size);
+            }
+            else
+            {
+                success = ReadParameterBuffer(static_cast<size_t>(header.data_size));
+            }
+
+            if (success)
+            {
+                for (auto decoder : decoders_)
+                {
+                    if (decoder->SupportsMetaDataId(meta_data_id))
+                    {
+                        decoder->DispatchInitSubresourceCommand(header, parameter_buffer_.data());
+                    }
+                }
+            }
+            else
+            {
+                if (format::IsBlockCompressed(block_header.type))
+                {
+                    HandleBlockReadError(kErrorReadingCompressedBlockData,
+                                         "Failed to read init subresource data meta-data block");
+                }
+                else
+                {
+                    HandleBlockReadError(kErrorReadingBlockData,
+                                         "Failed to read init subresource data meta-data block");
+                }
+            }
+        }
+        else
+        {
+            HandleBlockReadError(kErrorReadingBlockHeader,
+                                 "Failed to read init subresource data meta-data block header");
+        }
+    }
+    else if (meta_data_type == format::MetaDataType::kInitDx12AccelerationStructureCommand)
+    {
+        // Parse command header.
+        format::InitDx12AccelerationStructureCommandHeader header;
+        success = ReadBytes(&header.thread_id, sizeof(header.thread_id));
+        success = success &&
+                  ReadBytes(&header.dest_acceleration_structure_data, sizeof(header.dest_acceleration_structure_data));
+        success = success && ReadBytes(&header.copy_source_gpu_va, sizeof(header.copy_source_gpu_va));
+        success = success && ReadBytes(&header.copy_mode, sizeof(header.copy_mode));
+        success = success && ReadBytes(&header.inputs_type, sizeof(header.inputs_type));
+        success = success && ReadBytes(&header.inputs_flags, sizeof(header.inputs_flags));
+        success = success && ReadBytes(&header.inputs_num_instance_descs, sizeof(header.inputs_num_instance_descs));
+        success = success && ReadBytes(&header.inputs_num_geometry_descs, sizeof(header.inputs_num_geometry_descs));
+        success = success && ReadBytes(&header.inputs_data_size, sizeof(header.inputs_data_size));
+
+        // Parse geometry descs.
+        std::vector<format::InitDx12AccelerationStructureGeometryDesc> geom_descs;
+        if (success)
+        {
+            for (uint32_t i = 0; i < header.inputs_num_geometry_descs; ++i)
+            {
+                format::InitDx12AccelerationStructureGeometryDesc geom_desc;
+                success = success && ReadBytes(&geom_desc.geometry_type, sizeof(geom_desc.geometry_type));
+                success = success && ReadBytes(&geom_desc.geometry_flags, sizeof(geom_desc.geometry_flags));
+                success = success && ReadBytes(&geom_desc.aabbs_count, sizeof(geom_desc.aabbs_count));
+                success = success && ReadBytes(&geom_desc.aabbs_stride, sizeof(geom_desc.aabbs_stride));
+                success =
+                    success && ReadBytes(&geom_desc.triangles_has_transform, sizeof(geom_desc.triangles_has_transform));
+                success =
+                    success && ReadBytes(&geom_desc.triangles_index_format, sizeof(geom_desc.triangles_index_format));
+                success =
+                    success && ReadBytes(&geom_desc.triangles_vertex_format, sizeof(geom_desc.triangles_vertex_format));
+                success =
+                    success && ReadBytes(&geom_desc.triangles_index_count, sizeof(geom_desc.triangles_index_count));
+                success =
+                    success && ReadBytes(&geom_desc.triangles_vertex_count, sizeof(geom_desc.triangles_vertex_count));
+                success =
+                    success && ReadBytes(&geom_desc.triangles_vertex_stride, sizeof(geom_desc.triangles_vertex_stride));
+                geom_descs.push_back(geom_desc);
+            }
+        }
+
+        if (success)
+        {
+            if (header.inputs_data_size > 0)
+            {
+                GFXRECON_CHECK_CONVERSION_DATA_LOSS(size_t, header.inputs_data_size);
+
+                if (format::IsBlockCompressed(block_header.type))
+                {
+                    size_t uncompressed_size = 0;
+                    size_t compressed_size =
+                        static_cast<size_t>(block_header.size) -
+                        (sizeof(header) - sizeof(header.meta_header.block_header)) -
+                        (sizeof(format::InitDx12AccelerationStructureGeometryDesc) * header.inputs_num_geometry_descs);
+
+                    success = ReadCompressedParameterBuffer(
+                        compressed_size, static_cast<size_t>(header.inputs_data_size), &uncompressed_size);
+                }
+                else
+                {
+                    success = ReadParameterBuffer(static_cast<size_t>(header.inputs_data_size));
+                }
+            }
+
+            if (success)
+            {
+                for (auto decoder : decoders_)
+                {
+                    if (decoder->SupportsMetaDataId(meta_data_id))
+                    {
+                        decoder->DispatchInitDx12AccelerationStructureCommand(
+                            header, geom_descs, parameter_buffer_.data());
+                    }
+                }
+            }
+            else
+            {
+                HandleBlockReadError(kErrorReadingBlockData,
+                                     "Failed to read init DX12 acceleration structure meta-data block");
+            }
+        }
+        else
+        {
+            HandleBlockReadError(kErrorReadingBlockHeader,
+                                 "Failed to read init DX12 acceleration structure meta-data block header");
+        }
+    }
+    else if (meta_data_type == format::MetaDataType::kDxgiAdapterInfoCommand)
+    {
+        format::DxgiAdapterInfoCommandHeader adapter_info_header;
+        memset(&adapter_info_header, 0, sizeof(adapter_info_header));
+
+        success = ReadBytes(&adapter_info_header.thread_id, sizeof(adapter_info_header.thread_id));
+
+        success = success && ReadBytes(&adapter_info_header.adapter_desc.Description,
+                                       sizeof(adapter_info_header.adapter_desc.Description));
+        success = success && ReadBytes(&adapter_info_header.adapter_desc.VendorId,
+                                       sizeof(adapter_info_header.adapter_desc.VendorId));
+        success = success && ReadBytes(&adapter_info_header.adapter_desc.DeviceId,
+                                       sizeof(adapter_info_header.adapter_desc.DeviceId));
+        success = success && ReadBytes(&adapter_info_header.adapter_desc.SubSysId,
+                                       sizeof(adapter_info_header.adapter_desc.SubSysId));
+        success = success && ReadBytes(&adapter_info_header.adapter_desc.Revision,
+                                       sizeof(adapter_info_header.adapter_desc.Revision));
+        success = success && ReadBytes(&adapter_info_header.adapter_desc.DedicatedVideoMemory,
+                                       sizeof(adapter_info_header.adapter_desc.DedicatedVideoMemory));
+        success = success && ReadBytes(&adapter_info_header.adapter_desc.DedicatedSystemMemory,
+                                       sizeof(adapter_info_header.adapter_desc.DedicatedSystemMemory));
+        success = success && ReadBytes(&adapter_info_header.adapter_desc.SharedSystemMemory,
+                                       sizeof(adapter_info_header.adapter_desc.SharedSystemMemory));
+        success = success && ReadBytes(&adapter_info_header.adapter_desc.LuidLowPart,
+                                       sizeof(adapter_info_header.adapter_desc.LuidLowPart));
+        success = success && ReadBytes(&adapter_info_header.adapter_desc.LuidHighPart,
+                                       sizeof(adapter_info_header.adapter_desc.LuidHighPart));
+        success = success && ReadBytes(&adapter_info_header.adapter_desc.type, 
+                                       sizeof(adapter_info_header.adapter_desc.type));
+
+        if (success)
+        {
+            for (auto decoder : decoders_)
+            {
+                decoder->DispatchGetDxgiAdapterInfo(adapter_info_header);
+            }
+        }
+        else
+        {
+            HandleBlockReadError(kErrorReadingBlockData, "Failed to read adapter info meta-data block");
+        }
+    }
+    else if (meta_data_type == format::MetaDataType::kDx12RuntimeInfoCommand)
+    {
+        format::Dx12RuntimeInfoCommandHeader dx12_runtime_info_header;
+        memset(&dx12_runtime_info_header, 0, sizeof(dx12_runtime_info_header));
+
+        success = ReadBytes(&dx12_runtime_info_header.thread_id, sizeof(dx12_runtime_info_header.thread_id));
+
+        success = success && ReadBytes(&dx12_runtime_info_header.runtime_info.version,
+                                       sizeof(dx12_runtime_info_header.runtime_info.version));
+
+        success = success && ReadBytes(&dx12_runtime_info_header.runtime_info.src,
+                                       sizeof(dx12_runtime_info_header.runtime_info.src));
+
+        if (success)
+        {
+            for (auto decoder : decoders_)
+            {
+                decoder->DispatchGetDx12RuntimeInfo(dx12_runtime_info_header);
+            }
+        }
+        else
+        {
+            HandleBlockReadError(kErrorReadingBlockData, "Failed to read runtime info meta-data block");
+        }
+    }
     else
     {
         // Unrecognized metadata type.
@@ -1327,7 +1815,9 @@ bool FileProcessor::IsFrameDelimiter(format::ApiCallId call_id) const
 {
     // TODO: IDs of API calls that were treated as frame delimiters by the GFXReconstruct layer should be in the capture
     // file header.
-    return (call_id == format::ApiCallId::ApiCall_vkQueuePresentKHR);
+    return ((call_id == format::ApiCallId::ApiCall_vkQueuePresentKHR) ||
+            (call_id == format::ApiCallId::ApiCall_IDXGISwapChain_Present) ||
+            (call_id == format::ApiCallId::ApiCall_IDXGISwapChain1_Present1));
 }
 
 GFXRECON_END_NAMESPACE(decode)
