@@ -44,13 +44,17 @@ struct ScreenshotRange
 
 struct ReplayOptions
 {
-    bool enable_validation_layer{ false };
-    bool sync_queue_submissions{ false };
-    bool enable_debug_device_lost{ false };
-    bool create_dummy_allocations{ false };
-    bool omit_null_hardware_buffers{ false };
-    bool quit_after_measurement_frame_range{ false };
-    bool flush_measurement_frame_range{ false };
+    bool     enable_validation_layer{ false };
+    bool     sync_queue_submissions{ false };
+    bool     enable_debug_device_lost{ false };
+    bool     create_dummy_allocations{ false };
+    bool     omit_null_hardware_buffers{ false };
+    bool     quit_after_measurement_frame_range{ false };
+    bool     flush_measurement_frame_range{ false };
+    bool     force_windowed{ false };
+    uint32_t windowed_width{ 0 };
+    uint32_t windowed_height{ 0 };
+    int32_t  override_gpu_index{ -1 };
 };
 
 GFXRECON_END_NAMESPACE(decode)
