@@ -32,6 +32,7 @@
 #include "util/compressor.h"
 #include "util/file_path.h"
 #include "util/date_time.h"
+#include "util/driver_info.h"
 #include "util/logging.h"
 #include "util/page_guard_manager.h"
 #include "util/platform.h"
@@ -92,7 +93,8 @@ CaptureManager::CaptureManager(format::ApiFamilyId api_family) :
     page_guard_memory_mode_(kMemoryModeShadowInternal), trim_enabled_(false), trim_current_range_(0),
     current_frame_(kFirstFrame), capture_mode_(kModeWrite), previous_hotkey_state_(false),
     previous_runtime_trigger_state_(CaptureSettings::RuntimeTriggerState::kNotUsed), debug_layer_(false),
-    debug_device_lost_(false), screenshot_prefix_(""), screenshots_enabled_(false), global_frame_count_(0)
+    debug_device_lost_(false), screenshot_prefix_(""), screenshots_enabled_(false), global_frame_count_(0),
+    disable_dxr_(false), accel_struct_padding_(0), iunknown_wrapping_(false)
 {}
 
 CaptureManager::~CaptureManager()
@@ -242,6 +244,9 @@ bool CaptureManager::Initialize(std::string base_filename, const CaptureSettings
     screenshots_enabled_  = !trace_settings.screenshot_ranges.empty();
     screenshot_indices_   = CalcScreenshotIndices(trace_settings.screenshot_ranges);
     screenshot_prefix_    = PrepScreenshotPrefix(trace_settings.screenshot_dir);
+    disable_dxr_          = trace_settings.disable_dxr;
+    accel_struct_padding_ = trace_settings.accel_struct_padding;
+    iunknown_wrapping_    = trace_settings.iunknown_wrapping;
 
     if (memory_tracking_mode_ == CaptureSettings::kPageGuard)
     {
@@ -454,6 +459,66 @@ void CaptureManager::EndApiCallCapture()
     }
 }
 
+void CaptureManager::EndMethodCallCapture()
+{
+    if ((capture_mode_ & kModeWrite) == kModeWrite)
+    {
+        auto thread_data = GetThreadData();
+        assert(thread_data != nullptr);
+
+        auto parameter_buffer = thread_data->parameter_buffer_.get();
+        assert((parameter_buffer != nullptr) && (thread_data->parameter_encoder_ != nullptr));
+
+        bool   not_compressed    = true;
+        size_t uncompressed_size = parameter_buffer->GetDataSize();
+
+        if (compressor_ != nullptr)
+        {
+            size_t header_size     = sizeof(format::CompressedMethodCallHeader);
+            size_t compressed_size = compressor_->Compress(
+                uncompressed_size, parameter_buffer->GetData(), &thread_data->compressed_buffer_, header_size);
+
+            if ((compressed_size > 0) && (compressed_size < uncompressed_size))
+            {
+                auto compressed_header =
+                    reinterpret_cast<format::CompressedMethodCallHeader*>(thread_data->compressed_buffer_.data());
+                compressed_header->block_header.type = format::BlockType::kCompressedMethodCallBlock;
+                compressed_header->api_call_id       = thread_data->call_id_;
+                compressed_header->object_id         = thread_data->object_id_;
+                compressed_header->thread_id         = thread_data->thread_id_;
+                compressed_header->uncompressed_size = uncompressed_size;
+                compressed_header->block_header.size = sizeof(compressed_header->api_call_id) +
+                                                       sizeof(compressed_header->object_id) +
+                                                       sizeof(compressed_header->uncompressed_size) +
+                                                       sizeof(compressed_header->thread_id) + compressed_size;
+
+                WriteToFile(thread_data->compressed_buffer_.data(), header_size + compressed_size);
+
+                not_compressed = false;
+            }
+        }
+
+        if (not_compressed)
+        {
+            uint8_t* header_data = parameter_buffer->GetHeaderData();
+            assert((header_data != nullptr) &&
+                   (parameter_buffer->GetHeaderDataSize() == sizeof(format::MethodCallHeader)));
+
+            auto uncompressed_header               = reinterpret_cast<format::MethodCallHeader*>(header_data);
+            uncompressed_header->block_header.type = format::BlockType::kMethodCallBlock;
+            uncompressed_header->api_call_id       = thread_data->call_id_;
+            uncompressed_header->object_id         = thread_data->object_id_;
+            uncompressed_header->thread_id         = thread_data->thread_id_;
+            uncompressed_header->block_header.size = sizeof(uncompressed_header->api_call_id) +
+                                                     sizeof(uncompressed_header->object_id) +
+                                                     sizeof(uncompressed_header->thread_id) + uncompressed_size;
+
+            WriteToFile(parameter_buffer->GetHeaderData(),
+                        parameter_buffer->GetHeaderDataSize() + parameter_buffer->GetDataSize());
+        }
+    }
+}
+
 bool CaptureManager::IsTrimHotkeyPressed()
 {
     // Return true when GetKeyState() transitions from false to true
@@ -595,7 +660,7 @@ bool CaptureManager::ShouldTriggerScreenshot()
         uint32_t target_frame = screenshot_indices_.back();
 
         // If this is a frame of interest, take a screenshot
-        if (target_frame == global_frame_count_)
+        if (target_frame == (global_frame_count_ + 1))
         {
             triger_screenshot = true;
 
@@ -675,6 +740,11 @@ bool CaptureManager::CreateCaptureFile(const std::string& base_filename)
     {
         GFXRECON_LOG_INFO("Recording graphics API capture to %s", capture_filename.c_str());
         WriteFileHeader();
+
+        gfxrecon::util::filepath::FileInfo info{};
+        gfxrecon::util::filepath::GetApplicationInfo(info);
+        WriteExeFileInfo(info);
+
         // Save parameters of the capture in an annotation.
         std::string operation_annotation = "{\n"
                                            "    \"tool\": \"capture\",\n"
@@ -758,6 +828,21 @@ void CaptureManager::WriteDisplayMessageCmd(const char* message)
     }
 }
 
+void CaptureManager::WriteExeFileInfo(const gfxrecon::util::filepath::FileInfo& info)
+{
+    size_t                   info_length     = sizeof(format::ExeFileInfoBlock);
+    format::ExeFileInfoBlock exe_info_header = {};
+    exe_info_header.info_record              = info;
+
+    exe_info_header.meta_header.block_header.type = format::BlockType::kMetaDataBlock;
+    exe_info_header.meta_header.block_header.size = format::GetMetaDataBlockBaseSize(exe_info_header);
+    exe_info_header.meta_header.meta_data_id =
+        format::MakeMetaDataId(api_family_, format::MetaDataType::kExeFileInfoCommand);
+    exe_info_header.thread_id = GetThreadData()->thread_id_;
+
+    WriteToFile(&exe_info_header, sizeof(exe_info_header));
+}
+
 void CaptureManager::WriteAnnotation(const format::AnnotationType type, const char* label, const char* data)
 {
     if ((capture_mode_ & kModeWrite) == kModeWrite)
