@@ -22,8 +22,8 @@
 ** DEALINGS IN THE SOFTWARE.
 */
 
-#ifndef GFXRECON_ENCODE_CUSTOM_ENCODER_COMMANDS_H
-#define GFXRECON_ENCODE_CUSTOM_ENCODER_COMMANDS_H
+#ifndef GFXRECON_ENCODE_CUSTOM_VULKAN_ENCODER_COMMANDS_H
+#define GFXRECON_ENCODE_CUSTOM_VULKAN_ENCODER_COMMANDS_H
 
 #include "encode/vulkan_capture_manager.h"
 #include "format/api_call_id.h"
@@ -1143,4 +1143,4 @@ struct CustomEncoderPostCall<format::ApiCallId::ApiCall_vkSetLocalDimmingAMD>
 GFXRECON_END_NAMESPACE(encode)
 GFXRECON_END_NAMESPACE(gfxrecon)
 
-#endif // GFXRECON_ENCODE_CUSTOM_ENCODER_COMMANDS_H
+#endif // GFXRECON_ENCODE_CUSTOM_VULKAN_ENCODER_COMMANDS_H
