@@ -248,6 +248,8 @@ class VulkanStateTracker
         }
     }
 
+    void TrackTrimCommandPool(VkDevice device, VkCommandPool command_pool);
+
     void TrackResetCommandPool(VkCommandPool command_pool);
 
     void TrackPhysicalDeviceMemoryProperties(VkPhysicalDevice                        physical_device,
@@ -269,15 +271,22 @@ class VulkanStateTracker
 
     void TrackPhysicalDeviceSurfaceCapabilities(VkPhysicalDevice                physical_device,
                                                 VkSurfaceKHR                    surface,
-                                                const VkSurfaceCapabilitiesKHR& capabilities,
-                                                const void*                     surface_info_pnext = nullptr,
-                                                const void*                     capabilities_pnext = nullptr);
+                                                const VkSurfaceCapabilitiesKHR* capabilities);
+
+    void TrackPhysicalDeviceSurfaceCapabilities2(VkPhysicalDevice                       physical_device,
+                                                 const VkPhysicalDeviceSurfaceInfo2KHR& surface_info,
+                                                 VkSurfaceCapabilities2KHR*             surface_capabilities);
 
     void TrackPhysicalDeviceSurfaceFormats(VkPhysicalDevice          physical_device,
                                            VkSurfaceKHR              surface,
                                            uint32_t                  format_count,
                                            const VkSurfaceFormatKHR* formats);
 
+    void TrackPhysicalDeviceSurfaceFormats2(VkPhysicalDevice                       physical_device,
+                                            const VkPhysicalDeviceSurfaceInfo2KHR& surface_info,
+                                            uint32_t                               surface_format_count,
+                                            VkSurfaceFormat2KHR*                   surface_formats);
+
     void TrackPhysicalDeviceSurfacePresentModes(VkPhysicalDevice        physical_device,
                                                 VkSurfaceKHR            surface,
                                                 uint32_t                mode_count,
@@ -384,6 +393,14 @@ class VulkanStateTracker
 
     void TrackReleaseFullScreenExclusiveMode(VkDevice device, VkSwapchainKHR swapchain);
 
+    void TrackSetPrivateData(VkDevice          device,
+                             VkObjectType      objectType,
+                             uint64_t          objectHandle,
+                             VkPrivateDataSlot privateDataSlot,
+                             uint64_t          data);
+
+    void TrackSetLocalDimmingAMD(VkDevice device, VkSwapchainKHR swapChain, VkBool32 localDimmingEnable);
+
   private:
     template <typename ParentHandle, typename SecondaryHandle, typename Wrapper, typename CreateInfo>
     void AddGroupHandles(ParentHandle                  parent_handle,
