@@ -28,6 +28,8 @@
 #include "format/api_call_id.h"
 #include "util/compressor.h"
 #include "util/defines.h"
+#include "util/file_path.h"
+#include "util/driver_info.h"
 
 #include <cinttypes>
 #include <type_traits>
@@ -58,6 +60,7 @@ const uint32_t kCompressedBlockTypeBit    = 0x80000000;
 const size_t   kUuidSize                  = 16;
 const size_t   kMaxPhysicalDeviceNameSize = 256;
 const HandleId kNullHandleId              = 0;
+const size_t   kAdapterDescriptionSize    = 128;
 
 /// Label for operation annotation, which captures parameters used by tools
 /// operating on a capture file.
@@ -98,6 +101,13 @@ enum AnnotationType : uint32_t
     kXml     = 3
 };
 
+enum AdapterType : uint32_t
+{
+    kUnknownAdapter  = 0,
+    kSoftwareAdapter = 1,
+    kHardwareAdapter = 2
+};
+
 enum class MetaDataType : uint16_t
 {
     kUnknownMetaDataType                    = 0,
@@ -118,15 +128,15 @@ enum class MetaDataType : uint16_t
     kSetRayTracingShaderGroupHandlesCommand = 15,
     kCreateHeapAllocationCommand            = 16,
     kInitSubresourceCommand                 = 17,
-    kReserved18                             = 18,
-    kReserved19                             = 19,
-    kReserved20                             = 20,
-    kReserved21                             = 21,
-    kReserved22                             = 22,
+    kExeFileInfoCommand                     = 18,
+    kInitDx12AccelerationStructureCommand   = 19,
+    kFillMemoryResourceValueCommand         = 20,
+    kDxgiAdapterInfoCommand                 = 21,
+    kDriverInfoCommand                      = 22,
     kReserved23                             = 23,
     kCreateHardwareBufferCommand            = 24,
     kReserved25                             = 25,
-    kReserved26                             = 26,
+    kDx12RuntimeInfoCommand                 = 26,
 };
 
 // MetaDataId is stored in the capture file and its type must be uint32_t to avoid breaking capture file compatibility.
@@ -196,6 +206,16 @@ struct EnabledOptions
     CompressionType compression_type{ CompressionType::kNone };
 };
 
+// Resource values are values contained in resource data that may require special handling (e.g., mapping for replay).
+enum class ResourceValueType : uint8_t
+{
+    kUnknown                      = 0,
+    kGpuVirtualAddress            = 1,
+    kGpuDescriptorHandle          = 2,
+    kShaderIdentifier             = 3,
+    kIndirectArgumentDispatchRays = 4
+};
+
 #pragma pack(push)
 #pragma pack(4)
 
@@ -288,6 +308,17 @@ struct FillMemoryCommandHeader
     uint64_t memory_size;   // Uncompressed size of the data encoded after the header.
 };
 
+struct FillMemoryResourceValueCommandHeader
+{
+    MetaDataHeader   meta_header;
+    format::ThreadId thread_id; // thread_id is here as a placeholder. Currently always set to 0.
+    uint64_t         resource_value_count;
+
+    // The data for MapAndFillMemoryCommand will be organized as:
+    // map_value_count * ResourceValueType // type of value to map
+    // map_value_count * uint64_t          // offset of value to map
+};
+
 struct DisplayMessageCommandHeader
 {
     MetaDataHeader   meta_header;
@@ -297,6 +328,20 @@ struct DisplayMessageCommandHeader
     // terminator.
 };
 
+struct DriverInfoBlock
+{
+    MetaDataHeader   meta_header;
+    format::ThreadId thread_id;
+    char             driver_record[util::filepath::kMaxDriverInfoSize];
+};
+
+struct ExeFileInfoBlock
+{
+    MetaDataHeader              meta_header;
+    format::ThreadId            thread_id;
+    util::filepath::FileInfo    info_record;
+};
+
 // Not a header because this command does not include a variable length data payload.
 // All of the command data is present in the struct.
 struct ResizeWindowCommand
@@ -503,6 +548,74 @@ struct CreateHeapAllocationCommand
     uint64_t         allocation_size;
 };
 
+struct InitDx12AccelerationStructureCommandHeader
+{
+    MetaDataHeader meta_header{};
+    ThreadId       thread_id;
+    uint64_t       dest_acceleration_structure_data{ 0 };
+    uint64_t       copy_source_gpu_va{ 0 };
+    uint32_t       copy_mode{ 0 };
+    uint32_t       inputs_type{ 0 };
+    uint32_t       inputs_flags{ 0 };
+    uint32_t       inputs_num_instance_descs{ 0 }; ///< NumDescs for TLAS
+    uint32_t       inputs_num_geometry_descs{ 0 }; ///< NumDescs for BLAS
+    uint64_t       inputs_data_size{ 0 };
+
+    // In the capture file, accel struct data is written in the following order:
+    // InitDx12AccelerationStructureCommandHeader
+    // inputs_num_geometry_descs * InitDx12AccelerationStructureGeometryDesc
+    // build inputs data
+};
+
+struct InitDx12AccelerationStructureGeometryDesc
+{
+    uint32_t geometry_type{ 0 };
+    uint32_t geometry_flags{ 0 };
+    uint64_t aabbs_count{ 0 };
+    uint64_t aabbs_stride{ 0 };
+    uint32_t triangles_has_transform{ false };
+    uint32_t triangles_index_format{ 0 };
+    uint32_t triangles_vertex_format{ 0 };
+    uint32_t triangles_index_count{ 0 }; // 0 if no index buffer
+    uint32_t triangles_vertex_count{ 0 };
+    uint64_t triangles_vertex_stride{ 0 };
+};
+
+struct DxgiAdapterDesc
+{
+    wchar_t     Description[kAdapterDescriptionSize];
+    uint32_t    VendorId;
+    uint32_t    DeviceId;
+    uint32_t    SubSysId;
+    uint32_t    Revision;
+    uint64_t    DedicatedVideoMemory;
+    uint64_t    DedicatedSystemMemory;
+    uint64_t    SharedSystemMemory;
+    uint32_t    LuidLowPart;
+    int32_t     LuidHighPart;
+    AdapterType type;
+};
+
+struct DxgiAdapterInfoCommandHeader
+{
+    MetaDataHeader  meta_header;
+    ThreadId        thread_id;
+    DxgiAdapterDesc adapter_desc;
+};
+
+struct Dx12RuntimeInfo
+{
+    uint32_t version[util::filepath::kFileVersionSize] = {};
+    char     src[util::filepath::kMaxFilePropertySize] = {};
+};
+
+struct Dx12RuntimeInfoCommandHeader
+{
+    MetaDataHeader   meta_header;
+    format::ThreadId thread_id;
+    Dx12RuntimeInfo  runtime_info;
+};
+
 // Restore size_t to normal behavior.
 #undef size_t
 
