@@ -27,6 +27,7 @@
 
 #include "util/defines.h"
 #include "util/page_status_tracker.h"
+#include "util/platform.h"
 
 #include <cstddef>
 #include <cstdint>
@@ -36,16 +37,22 @@
 #include <unordered_map>
 #include <vector>
 
+#if !defined(WIN32)
+#include <pthread.h>
+#endif
+
 GFXRECON_BEGIN_NAMESPACE(gfxrecon)
 GFXRECON_BEGIN_NAMESPACE(util)
 
 class PageGuardManager
 {
   public:
-    static const bool kDefaultEnableCopyOnMap         = true;
-    static const bool kDefaultEnableSeparateRead      = true;
-    static const bool kDefaultEnableReadWriteSamePage = true;
-    static const bool kDefaultUnblockSIGSEGV          = false;
+    static const bool kDefaultEnableCopyOnMap                 = true;
+    static const bool kDefaultEnableSeparateRead              = true;
+    static const bool kDefaultEnableReadWriteSamePage         = true;
+    static const bool kDefaultUnblockSIGSEGV                  = false;
+    static const bool kDefaultEnableSignalHandlerWatcher      = false;
+    static const int  kDefaultSignalHandlerWatcherMaxRestores = 1;
 
     static const uintptr_t kNullShadowHandle = 0;
 
@@ -56,8 +63,12 @@ class PageGuardManager
     typedef std::function<void(uint64_t, void*, size_t, size_t)> ModifiedMemoryFunc;
 
   public:
-    static void
-    Create(bool enable_copy_on_map, bool enable_separate_read, bool expect_read_write_same_page, bool unblock_SIGSEGV);
+    static void Create(bool enable_copy_on_map,
+                       bool enable_separate_read,
+                       bool expect_read_write_same_page,
+                       bool unblock_SIGSEGV,
+                       bool enable_signal_handler_watcher,
+                       int  signal_handler_watcher_max_restores);
 
     static void Destroy();
 
@@ -112,7 +123,9 @@ class PageGuardManager
     PageGuardManager(bool enable_copy_on_map,
                      bool enable_separate_read,
                      bool expect_read_write_same_page,
-                     bool unblock_SIGSEGV);
+                     bool unblock_SIGSEGV,
+                     bool enable_signal_handler_watcher,
+                     int  signal_handler_watcher_max_restores);
 
     ~PageGuardManager();
 
@@ -184,7 +197,6 @@ class PageGuardManager
     typedef std::unordered_map<uint64_t, MemoryInfo> MemoryInfoMap;
 
   private:
-    size_t GetSystemPageSize() const;
     size_t GetSystemPagePotShift() const;
     void   InitializeSystemExceptionContext();
 
@@ -218,6 +230,7 @@ class PageGuardManager
     static PageGuardManager* instance_;
     MemoryInfoMap            memory_info_;
     std::mutex               tracked_memory_lock_;
+    std::mutex               signal_handler_lock_;
     void*                    exception_handler_;
     uint32_t                 exception_handler_count_;
     const size_t             system_page_size_;
@@ -225,9 +238,21 @@ class PageGuardManager
     const bool               enable_copy_on_map_;
     const bool               enable_separate_read_;
     const bool               unblock_sigsegv_;
+    bool                     enable_signal_handler_watcher_;
+    int                      signal_handler_watcher_max_restores_;
 
     // Only applies to WIN32 builds and Linux/Android builds with PAGE_GUARD_ENABLE_UCONTEXT_WRITE_DETECTION defined.
     const bool enable_read_write_same_page_;
+
+#if !defined(WIN32)
+    pthread_t signal_handler_watcher_thread_;
+
+    static void* SignalHandlerWatcher(void*);
+    static bool  CheckSignalHandler();
+    void         MarkAllTrackedMemoryAsDirty();
+
+    static uint32_t signal_handler_watcher_restores_;
+#endif
 };
 
 GFXRECON_END_NAMESPACE(util)
